from typing import Any, Dict, List

from sktmls.models import MLSModelError, MLSRuleModel


class SampleRuleModel(MLSRuleModel):
    def predict(self, x: List[Any]) -> Dict[str, Any]:
        age = x[0]
        app_use_traffic_youtube = x[1]
        svc_scrb_period = x[2]

        if len(self.features) != len(x):
            raise MLSModelError("The length of input is different from features on model.json")

        if svc_scrb_period < 100 or age < 19 or age > 60 or app_use_traffic_youtube < 10:
            return {"items": []}
        else:
            return {
                "items": [{"id": "NA00006538", "name": "T플랜 스페셜", "props": {"percent_rk": "0.5755"}, "type": "fee"}]
            }



from typing import Any, Dict, List

from sktmls.models import MLSModelError, MLSRuleModel


class BnfLoyaltyRuleModel(MLSRuleModel):
    def predict(self, x: List[Any]) -> Dict[str, Any]:
        if len(self.features) != len(x):
            raise MLSModelError("The length of input is different from that of features in model.json")

        svc_scrb_period = x[0]
        if svc_scrb_period <= 730:
            return {"items": []}
        else:
            return {"items": [{"id": "BNF0000003", "name": "장기 고객 혜택 정보", "type": "bnf_loyalty", "props": {}}]}



import os
import sys
import importlib

MLS_RUNTIME_ENV = os.environ.get("MLS_RUNTIME_ENV")

if MLS_RUNTIME_ENV and MLS_RUNTIME_ENV not in ["YE", "EDD", "LOCAL"]:
    raise Exception(f"Unsupported MLS_RUNTIME_ENV: {MLS_RUNTIME_ENV}")
else:
    __HOSTNAME = os.environ.get("HOSTNAME", "").lower()

    if __HOSTNAME.startswith("bdp-dmi"):
        MLS_RUNTIME_ENV = "YE"
    elif __HOSTNAME.startswith("vm-skt"):
        MLS_RUNTIME_ENV = "EDD"
    else:
        MLS_RUNTIME_ENV = "LOCAL"

importlib.import_module(f"sktmls.config.{MLS_RUNTIME_ENV.lower()}")


class Config:
    def __init__(self, runtime_env: str):
        setattr(self, "MLS_RUNTIME_ENV", MLS_RUNTIME_ENV)

        module = sys.modules[f"sktmls.config.{runtime_env}"]
        for key in dir(module):
            if key.isupper():
                setattr(self, key, getattr(module, key))


config = Config(MLS_RUNTIME_ENV.lower())



from typing import Any, Dict, List

import numpy as np

from sktmls.models import MLSLightGBMModel


class SampleModel(MLSLightGBMModel):
    def predict(self, x: List[Any]) -> Dict[str, Any]:
        if not self.model:
            return {"items": []}

        y = self.model.predict(np.array([x])).flatten().tolist()[0]

        if y < 0.2:
            return {"items": []}

        return {
            "items": [
                {
                    "id": "BNF0000001",
                    "name": "T건강습관",
                    "props": {"context_id": "context_discount_benefit", "score": y},
                    "type": "bnf_healthhabit",
                }
            ]
        }



from .mls_env import MLSENV
from .model_registry import ModelRegistry, ModelRegistryError
from .mls_client import MLSClient, MLSResponse, MLSClientError

__all__ = ["MLSENV", "ModelRegistry", "ModelRegistryError", "MLSClient", "MLSResponse", "MLSClientError"]



from .default_model import DefaultLightGBMModel, DefaultXGBoostModel
from .sample_model import SampleModel
from .sample_rule_model import SampleRuleModel
from .random_pick_model import RandomPickModel, PeriodicRandomPickModel
from .bnf_loyalty_rule_model import BnfLoyaltyRuleModel
from .info_unpaid_rule_model import InfoUnpaidRuleModel
from .mbr_vip_info_rule_model import MbrVipInfoRuleModel
from .info_defect_rule_model import InfoDefectRuleModel
from .shuffle_list_model import ShuffleListModel, PeriodicShuffleListModel

__all__ = [
    "DefaultLightGBMModel",
    "DefaultXGBoostModel",
    "SampleModel",
    "SampleRuleModel",
    "RandomPickModel",
    "PeriodicRandomPickModel",
    "BnfLoyaltyRuleModel",
    "InfoUnpaidRuleModel",
    "MbrVipInfoRuleModel",
    "InfoDefectRuleModel",
    "ShuffleListModel",
    "PeriodicShuffleListModel",
]



from typing import List, Union
from enum import Enum

from dateutil import parser

from sktmls import MLSClient, MLSENV, MLSClientError, MLSResponse
from sktmls.datasets import Dataset

MLS_MODELS_API_URL = "/api/v1/models"

MODEL_LIBS = ["sklearn", "xgboost", "lightgbm", "rule", "etc"]


class MLModel:
    """
    ML모델 클래스 입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) ML모델 고유 ID
            - name: (str) ML모델 이름
            - version: (str) ML모델 버전
            - creator: (str) ML모델 생성 계정명
            - description: (str) ML모델 설명
            - model_type: (str) ML모델 타입 (`automl` | `manual`)
            - table: (str) ML모델 테이블
            - model_data: (str) ML모델 데이터
            - status: (`sktmls.models.MLModelStatus`) ML모델 상태
            - created_at: (datetime) 생성일시
            - updated_at: (datetime) 수정일시

        ## Returns
        `sktmls.models.MLModel`
        """
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.version = kwargs.get("version")
        self.description = kwargs.get("description")
        self.creator = kwargs.get("creator")
        self.model_type = kwargs.get("model_type")
        self.table = kwargs.get("table")
        self.model_data = kwargs.get("model_data")
        self.status = kwargs.get("status")
        try:
            self.created_at = parser.parse(kwargs.get("created_at"))
        except TypeError:
            self.created_at = None
        try:
            self.updated_at = parser.parse(kwargs.get("updated_at"))
        except TypeError:
            self.updated_at = None

    def __str__(self) -> str:
        return self.name

    def get(self) -> dict:
        return self.__dict__

    def reset(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class AutoMLModel(MLModel):
    """
    AutoML모델 클래스 입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) ML모델 고유 ID
            - name: (str) ML모델 이름
            - model_type: (str) ML모델 타입 (`automl` | `manual`)
            - version: (str) ML모델 버전
            - description: (str) ML모델 설명
            - creator: (str) ML모델 생성 계정명
            - status: (`sktmls.models.MLModelStatus`) ML모델 상태
            - model_meta: (dict) ML모델 메타정보
            - components: (dict) ML모델 컴포넌트
            - dataset_id: (int) ML 데이터셋 고유 ID
            - dataset_name: (str) ML 데이터셋 이름
            - automl_model_info: (dict) AutoML모델 정보

        ## Returns
        `sktmls.models.AutoMLModel`
        """

        super().__init__(**kwargs)
        self.dataset_id = kwargs.get("dataset_id")
        self.dataset_name = kwargs.get("dataset_name")
        self.automl_model_info = kwargs.get("automl_model_info")


class ManualModel(MLModel):
    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) ML모델 고유 ID
            - name: (str) ML모델 이름
            - model_type: (str) ML모델 타입 (`automl` | `manual`)
            - version: (str) ML모델 버전
            - description: (str) ML모델 설명
            - creator: (str) ML모델 생성 계정명
            - status: (`sktmls.models.MLModelStatus`) ML모델 상태
            - model_meta: (dict) ML모델 메타정보
            - components: (dict) ML모델 컴포넌트
            - table: (str) ML모델 테이블
            - model_lib: (str) ML모델 라이브러리
            - model_data: (str) ML모델 데이터
            - features: (str) ML모델에 사용된 피쳐
            - is_enabled: (bool) 활성상태 여부

        ## Returns
        `sktmls.models.ManualModel`
        """

        super().__init__(**kwargs)
        self.model_meta = kwargs.get("model_meta")
        self.components = kwargs.get("components")
        self.model_lib = kwargs.get("model_lib")
        self.features = kwargs.get("features")
        self.is_enabled = kwargs.get("is_enabled")


class MLModelStatus(Enum):
    IN_USE = "IN_USE"
    NOT_IN_USE = "NOT_IN_USE"
    AUTOML_TRAINING = "TRAINING"
    AUTOML_DONE = "DONE"
    AUTOML_FAILED = "FAILED"
    UNKNOWN = "UNKNOWN"


class MLModelClient(MLSClient):
    def __init__(self, env: MLSENV = None, username: str = None, password: str = None):
        """
        ## Args

        - env: (`sktmls.MLSENV`) 접근할 MLS 환경 (`sktmls.MLSENV.DEV`|`sktmls.MLSENV.STG`|`sktmls.MLSENV.PRD`) (기본값: `sktmls.MLSENV.STG`)
        - username: (str) MLS 계정명 (기본값: $MLS_USERNAME)
        - password: (str) MLS 계정 비밀번호 (기본값: $MLS_PASSWORD)

        ## Returns
        `sktmls.models.MLModelClient`

        ## Example

        ```
        ml_model_client = MLModelClient(env=MLSENV.STG, username="mls_account", password="mls_password")
        ```
        """
        super().__init__(env=env, username=username, password=password)

    def create_automl_model(self, dataset: Dataset, name: str, version: str, description: str) -> AutoMLModel:
        """
        AutoML 모델을 생성합니다.

        ## Args

        - dataset: (sktmls.datasets.Dataset) ML 데이터셋
        - name: (str) 생성할 AutoML 모델 이름
        - version: (str) 생성할 AutoML 모델 버전
        - description: (str) 생성할 AutoML 설명

        ## Returns
        `sktmls.models.AutoMLModel`

        - name: (str) ML모델 이름
        - model_type: (str) ML모델 타입 (`automl` | `manual`)
        - version: (str) ML모델 버전
        - creator: (str) ML모델 생성 계정명
        - description: (str) ML모델 설명
        - status: (`sktmls.models.MLModelStatus`) ML모델 상태
        - model_meta: (dict) ML모델 메타정보
        - components: (dict) ML모델 컴포넌트
        - dataset_id: (int) ML 데이터셋 고유 ID
        - dataset_name: (str) ML 데이터셋 이름
        - automl_model_info: (dict) AutoML모델 정보

        ## Example

        ```
        automl_model = ml_model_client.create_automl_model(
            dataset=dataset,
            name="automl_test_model",
            version="v1",
            description='test_model'
        )
        ```
        """
        assert type(dataset) == Dataset

        data = {
            "model_type": "automl",
            "name": name,
            "version": version,
            "description": description,
            "dataset_id": dataset.id,
        }
        response = self._request(method="POST", url=MLS_MODELS_API_URL, data=data).results
        return self.get_automl_model(id=response.get("id"))

    def create_manual_model(
        self,
        name: str,
        version: str,
        description: str = None,
        creator: str = None,
        model_lib: str = None,
        model_data: str = None,
        features: str = None,
        table: str = None,
        components: dict = None,
    ) -> ManualModel:
        """
        일반 모델을 생성합니다.

        ## Args

        - name: (str) 생성할 일반 모델 이름
        - version: (str) 생성할 일반 모델 버전
        - description: (optional) (str) 생성할 일반 모델 설명
        - creator: (optional) (str) 일반 모델 생성 계정명
        - model_lib: (optional) (str) 일반 모델 라이브러리
        - model_data: (optional) (str) 일반 모델 데이터
        - features: (optional) (str) 일반 모델에 사용된 피쳐
        - table: (optional) (str) 일반 모델 테이블
        - components: (optional) (dict) 일반 모델 컴포넌트

        ## Returns
        `sktmls.models.ManualModel`

        - id: (int) ML모델 고유 ID
        - name: (str) ML모델 이름
        - model_type: (str) ML모델 타입 (`automl` | `manual`)
        - version: (str) ML모델 버전
        - description: (str) ML모델 설명
        - creator: (str) ML모델 생성 계정명
        - status: (`sktmls.models.MLModelStatus`) ML모델 상태
        - model_meta: (dict) ML모델 메타정보
        - components: (dict) ML모델 컴포넌트
        - table: (str) ML모델 테이블
        - model_lib: (str) ML모델 라이브러리
        - model_data: (str) ML모델 데이터
        - features: (str) ML모델에 사용된 피쳐
        - is_enabled: (bool) 활성상태 여부

        ## Example

        ```
        manual_model = ml_model_client.create_manual_model(
            model_name="manual_test_model",
            model_version="v1",
            description="test_model",
            creator="mls_user",
            model_lib="lightgbm",
            model_data="/models/manual_test_model/v1/model.joblib",
            features="age,app_use_traffic_youtube,svc_scrb_period",
            table="test_model_table",
            components={
                "id": 3,
                "name": "test_component",
                "info": "test",
                "is_latest": True,
                "user": "mls_user"
            },
        )

        ```
        """

        data = {
            "model_type": "manual",
            "name": name,
            "version": version,
        }
        if description:
            data["description"] = description
        if creator:
            data["creator"] = creator
        if model_lib:
            assert model_lib in MODEL_LIBS, "허용되지 않는 `model_lib`값입니다."
            data["model_lib"] = model_lib
        if model_data:
            data["model_data"] = model_data
        if features:
            data["features"] = features
        if table:
            data["table"] = table
        if components:
            assert type(components) == dict, "components는 dict 타입이어야 합니다."
            data["components"] = [components]

        response = self._request(method="POST", url=MLS_MODELS_API_URL, data=data).results
        return self.get_manual_model(id=response.get("id"))

    def list_ml_models(self, **kwargs) -> List[MLModel]:
        """
        ML모델 리스트를 가져옵니다.

        ## Args

        - kwargs: (optional) (dict) 필터 조건
            - id: (int) ML모델 고유 ID
            - name: (str) ML모델 이름
            - version: (str) ML모델 이름

        ## Returns
        list(`sktmls.models.MLModel`)

        - id: (int) ML모델 고유 ID
        - name: (str) ML모델 이름
        - version: (str) ML모델 버전
        - creator: (str) ML모델 생성 계정명
        - model_type: (str) ML모델 타입 (`automl` | `manual`)
        - table: (str) ML모델 테이블
        - model_data: (str) ML모델 데이터
        - status: (`sktmls.models.MLModelStatus`) ML모델 상태
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```
        ml_models = ml_model_client.list_ml_models()
        ```
        """
        response = self._request(method="GET", url=MLS_MODELS_API_URL, params=kwargs)
        return [MLModel(**ml_model) for ml_model in response.results]

    def get_automl_model(self, id: int = None, name: str = None, version: str = None) -> AutoMLModel:
        """
        AutoML모델 정보를 가져옵니다.

        ## Args: `id` 또는 (`name` and `version`) 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - id: (int) ML모델 고유 ID
        - name: (str) ML모델 이름
        - version: (str) ML모델 이름

        ## Returns
        `sktmls.models.AutoMLModel`

        - id: (int) ML모델 고유 ID
        - name: (str) ML모델 이름
        - model_type: (str) ML모델 타입 (`automl` | `manual`)
        - version: (str) ML모델 버전
        - description: (str) ML모델 설명
        - creator: (str) ML모델 생성 계정명
        - status: (`sktmls.models.MLModelStatus`) ML모델 상태
        - model_meta: (dict) ML모델 메타정보
        - components: (dict) ML모델 컴포넌트
        - dataset_id: (int) ML 데이터셋 고유 ID
        - dataset_name: (str) ML 데이터셋 이름
        - automl_model_info: (dict) AutoML모델 정보

        ## Example

        ```
        ml_model_by_id = ml_model_client.get_automl_model(id=3)
        ml_model_by_name_and_version = ml_model_client.get_automl_model(name="my_automl_model", version="my_automl_version")
        ```
        """
        assert id or (name and version), "`id` 또는 (`name`, `version`) 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        ml_models = self.list_ml_models(id=id, name=name, version=version)
        if len(ml_models) == 0:
            raise MLSClientError(code=404, msg="ML모델이 없습니다.")
        elif len(ml_models) > 1:
            raise MLSClientError(code=409, msg="같은 이름의 모델이 여러개 존재합니다.")

        if ml_models[0].model_type != "automl":
            raise MLSClientError(code=400, msg="AutoML 모델이 아닙니다.")

        return AutoMLModel(**self._request(method="GET", url=f"{MLS_MODELS_API_URL}/{ml_models[0].id}").results)

    def get_manual_model(self, id: int = None, name: str = None, version: str = None) -> ManualModel:
        """
        일반 모델 정보를 가져옵니다.

        ## Args: `id` 또는 (`name` and `version`) 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - id: (int) ML모델 고유 ID
        - name: (str) ML모델 이름
        - version: (str) ML모델 이름

        ## Returns
        `sktmls.models.ManualModel`

        - id: (int) ML모델 고유 ID
        - name: (str) ML모델 이름
        - model_type: (str) ML모델 타입 (`automl` | `manual`)
        - version: (str) ML모델 버전
        - description: (str) ML모델 설명
        - creator: (str) ML모델 생성 계정명
        - status: (`sktmls.models.MLModelStatus`) ML모델 상태
        - model_meta: (dict) ML모델 메타정보
        - components: (dict) ML모델 컴포넌트
        - table: (str) ML모델 테이블
        - model_lib: (str) ML모델 라이브러리
        - model_data: (str) ML모델 데이터
        - features: (str) ML모델에 사용된 피쳐
        - is_enabled: (bool) 활성상태 여부

        ## Example

        ```
        ml_model_by_id = ml_model_client.get_manual_model(id=3)
        ml_model_by_name_and_version = ml_model_client.get_manual_model(name="my_manual_model", version="my_manual_version")
        ```
        """
        assert id or (name and version), "`id` 또는 (`name`, `version`) 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        ml_models = self.list_ml_models(id=id, name=name, version=version)
        if len(ml_models) == 0:
            raise MLSClientError(code=404, msg="ML모델이 없습니다.")
        elif len(ml_models) > 1:
            raise MLSClientError(code=409, msg="같은 이름의 모델이 여러개 존재합니다.")

        if ml_models[0].model_type != "manual":
            raise MLSClientError(code=400, msg="일반 모델이 아닙니다.")

        return ManualModel(**self._request(method="GET", url=f"{MLS_MODELS_API_URL}/{ml_models[0].id}").results)

    def update_manual_model(
        self,
        manual_model: ManualModel,
        name: str = None,
        version: str = None,
        description: str = None,
        creator: str = None,
        model_lib: str = None,
        model_data: str = None,
        features: str = None,
        table: str = None,
        components: dict = None,
    ) -> ManualModel:
        """
        일반 모델 정보를 수정합니다.

        ## Args

        - model: (`sktmls.models.ManualModel`) 일반 모델 객체
        - name: (str) (optional) 생성할 일반 모델 이름
        - version: (str) (optional) 생성할 일반 모델 버전
        - description: (optional) (str) 생성할 일반 모델 설명
        - creator: (optional) (str) 일반 모델 생성 계정명
        - model_lib: (optional) (str) 일반 모델 라이브러리
        - model_data: (optional) (str) 일반 모델 데이터
        - features: (optional) (str) 일반 모델에 사용된 피쳐
        - table: (optional) (str) 일반 모델 테이블
        - components: (optional) (dict) 일반 모델 컴포넌트

        ## Returns
        `sktmls.models.ManualModel`

        - id: (int) ML모델 고유 ID
        - name: (str) ML모델 이름
        - model_type: (str) ML모델 타입 (`automl` | `manual`)
        - version: (str) ML모델 버전
        - description: (str) ML모델 설명
        - creator: (str) ML모델 생성 계정명
        - status: (`sktmls.models.MLModelStatus`) ML모델 상태
        - model_meta: (dict) ML모델 메타정보
        - components: (dict) ML모델 컴포넌트
        - table: (str) ML모델 테이블
        - model_lib: (str) ML모델 라이브러리
        - model_data: (str) ML모델 데이터
        - features: (str) ML모델에 사용된 피쳐
        - is_enabled: (bool) 활성상태 여부

        ## Example

        ```
        manual_model = model_client.get_manual_model(name="my_manual_model")
        manual_model = model_client.update_manual_model(
            model=manual_model,
            name="my_new_manual_model"
            description="updated manual model",
            model_data="/models/my_new_manual_model/v1/model.joblib",
        )
        ```
        """
        assert type(manual_model) == ManualModel

        data = manual_model.get()
        if name:
            data["name"] = name
        if version:
            data["version"] = version
        if description:
            data["description"] = description
        if creator:
            data["creator"] = creator
        if model_lib:
            assert model_lib in MODEL_LIBS, "허용되지 않는 `model_lib`값입니다."
            data["model_lib"] = model_lib
        if model_data:
            data["model_data"] = model_data
        if features:
            data["features"] = features
        if table:
            data["table"] = table
        if components:
            assert type(components) == dict, "components는 dict 타입이어야 합니다."
            data["components"] = [components]

        manual_model.reset(
            **self._request(method="PUT", url=f"{MLS_MODELS_API_URL}/{manual_model.id}", data=data).results
        )
        return manual_model

    def delete_model(self, model: Union[AutoMLModel, ManualModel]) -> MLSResponse:
        """
        모델 삭제합니다.

        ## Args

        - model: (`sktmls.models.AutoMLModel` or `sktmls.models.ManualModel`) AutoML 혹은 일반 모델 객체

        ## Returns
        `sktmls.MLSResponse`

        ## Example

        ```
        model_client.delete_model(model)
        ```
        """

        assert type(model) in [
            AutoMLModel,
            ManualModel,
        ], "`model`은 `sktmls.models.AutoMLModel` 혹은 `sktmls.models.ManualModel` 타입이어야 합니다."

        return self._request(method="DELETE", url=f"{MLS_MODELS_API_URL}/{model.id}")



from typing import Dict, Any, List

import pandas
from dateutil import parser

from sktmls import MLSClient, MLSENV, MLSResponse, MLSClientError

MLS_META_API_URL = "/api/v1/meta_tables"


class MetaTable:
    """
    MLS 메타 테이블 클래스입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) 메타 테이블 고유 ID
            - name: (str) 메타 테이블 이름
            - description: (str) 메타 테이블 설명
            - schema: (dict) 메타 아이템 스키마
            - items: (list) 포함된 메타 아이템 리스트
            - created_at: (datetime) 생성일시
            - updated_at: (datetime) 수정일시
            - user: (str) 메타 테이블 소유 계정명

        ## Returns
        `sktmls.meta_tables.MetaTable`
        """
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.description = kwargs.get("description")
        self.schema = kwargs.get("schema")
        self.items = kwargs.get("items")
        self.created_at = parser.parse(kwargs.get("created_at"))
        self.updated_at = parser.parse(kwargs.get("updated_at"))
        self.user = kwargs.get("user")

    def get(self):
        return self.__dict__

    def reset(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class MetaItem:
    """
    MLS 메타 아이템 클래스입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) 메타 아이템 고유 ID
            - name: (str) 메타 아이템 이름
            - values: (dict) 메타 아이템 정보

        ## Returns
        `sktmls.meta_tables.MetaItem`
        """
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.values = kwargs.get("values")

    def get(self):
        return self.__dict__

    def reset(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class MetaTableClient(MLSClient):
    """
    MLS 메타 테이블과 메타 아이템 관련 기능들을 제공하는 클라이언트입니다.
    """

    def __init__(
        self, env: MLSENV = None, username: str = None, password: str = None,
    ):
        """
        ## Args

        - env: (`sktmls.MLSENV`) 접근할 MLS 환경 (`sktmls.MLSENV.DEV`|`sktmls.MLSENV.STG`|`sktmls.MLSENV.PRD`) (기본값: `sktmls.MLSENV.STG`)
        - username: (str) MLS 계정명 (기본값: $MLS_USERNAME)
        - password: (str) MLS 계정 비밀번호 (기본값: $MLS_PASSWORD)

        ## Returns
        `sktmls.meta_tables.MetaTableClient`

        ## Example

        ```
        meta_table_client = MetaTableClient(env=MLSENV.STG, username="mls_account", password="mls_password")
        ```
        """

        super().__init__(env=env, username=username, password=password)

    def create_meta_table(self, name: str, schema: Dict[str, Dict], description: str = None) -> MetaTable:
        """
        메타 테이블을 생성합니다.

        ## Args

        - name: (str) 메타 테이블 이름
        - schema: (dict) 메타 아이템 스키마
        - description: (optional) (str) 메타 테이블 설명

        ## Returns
        `sktmls.meta_tables.MetaTable`

        - id: (int) 생성된 메타 테이블 고유 ID
        - name: (str) 생성된 메타 테이블 이름
        - description: (str) 생성된 메타 테이블 설명
        - schema: (dict) 생성된 메타 아이템 스키마
        - items: (list) 포함된 메타 아이템 리스트
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시
        - user: (str) 메타 테이블 소유 계정명

        ## Example

        ```
        meta_table = meta_table_client.create_meta_table(
            name="my_meta_table",
            schema={
                "meta_item_schema1": {"type": "string"},
                "meta_item_schema2": {"type": "number"}
            },
            description="This is my table"
        )
        ```
        """

        data = {
            "name": name,
            "schema": schema,
        }
        if description:
            data["description"] = description

        return MetaTable(**self._request(method="POST", url=MLS_META_API_URL, data=data).results)

    def update_meta_table(
        self, meta_table: MetaTable, name: str = None, schema: Dict[str, Dict] = None, description: str = None
    ) -> MetaTable:
        """
        메타 테이블 정보를 수정합니다.

        ## Args

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 테이블 객체
        - name: (optional) (str) 메타 테이블 이름
        - schema: (optional) (dict) 메타 아이템 스키마
        - description: (optional) (str) 메타 테이블 설명

        ## Returns
        `sktmls.meta_tables.MetaTable`

        - id: (int) 수정된 메타 테이블 고유 ID
        - name: (str) 수정된 메타 테이블 이름
        - description: (str) 수정된 메타 테이블 설명
        - schema: (dict) 수정된 메타 아이템 스키마
        - items: (list) 포함된 메타 아이템 리스트
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시
        - user: (str) 메타 테이블 소유 계정명

        ## Example

        ```
        meta_table = meta_table_client.update_meta_table(
            meta_table=meta_table
            name="my_meta_table",
            schema={
                "meta_item_schema3": {"type": "string"},
                "meta_item_schema4": {"type": "number"}
            },
            description="This is my updated table"
        )
        ```
        """
        assert type(meta_table) == MetaTable

        data = meta_table.get()
        if name:
            data["name"] = name
        if schema:
            data["schema"] = schema
        if description:
            data["description"] = description

        meta_table.reset(**self._request(method="PUT", url=f"{MLS_META_API_URL}/{meta_table.id}", data=data).results)

        return meta_table

    def get_meta_table(self, id: int = None, name: str = None) -> MetaTable:
        """
        메타 테이블 정보를 가져옵니다.

        ## Args: `id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - id: (optional) (int) 메타 테이블 고유 ID
        - name: (optional) (str) 메타 테이블 이름

        ## Returns
        `sktmls.meta_tables.MetaTable`

        - id: (int) 메타 테이블 고유 ID
        - name: (str) 메타 테이블 이름
        - description: (str) 메타 테이블 설명
        - schema: (dict) 메타 아이템 스키마
        - items: (list) 포함된 메타 아이템 리스트
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시
        - user: (str) 메타 테이블 소유 계정명

        ## Example

        ```
        meta_table_by_id = meta_table_client.get_meta_table(id=3)
        meta_table_by_name = meta_table_client.get_meta_table(name="my_table")
        ```
        """
        assert id or name, "`id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        meta_tables = self.list_meta_tables(id=id, name=name)
        if len(meta_tables) == 0:
            raise MLSClientError(code=404, msg="메타 테이블이 없습니다.")
        elif len(meta_tables) > 1:
            raise MLSClientError(code=409, msg="메타 테이블이 여러개 존재합니다.")
        return meta_tables[0]

    def get_meta_table_dataframe(self, id: int = None, name: str = None) -> pandas.core.frame.DataFrame:
        """
        메타 테이블 정보를 가져와서 `pandas.core.frame.DataFrame` 형태로 반환합니다.

        ## Args: `id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - id: (optional) (int) 메타 테이블 고유 ID
        - name: (optional) (str) 메타 테이블 이름

        ## Returns
        `pandas.core.frame.DataFrame`

        ## Example

        ```
        meta_table_df_by_id = meta_table_client.get_meta_table_dataframe(id=3)
        meta_table_df_by_name = meta_table_client.get_meta_table_dataframe(name="my_table")
        ```
        """
        assert id or name, "`id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        results = self.get_meta_table(id=id, name=name).get()
        items = results.get("items")

        if items:
            key = pandas.DataFrame.from_records(items)["name"]
            values = pandas.DataFrame.from_records(pandas.DataFrame.from_records(items)["values"])

            return pandas.concat([key, values], axis=1)

    def list_meta_tables(self, **kwargs) -> List[MetaTable]:
        """
        메타 테이블 리스트를 가져옵니다.

        ## Args

        - kwargs: (optional) (dict) 필터 조건
            - id: (int) 메타 테이블 고유 ID
            - name: (str) 메타 테이블 이름

        ## Returns
        list(`sktmls.meta_tables.MetaTable`)

        - id: (int) 메타 테이블 고유 ID
        - name: (str) 메타 테이블 이름
        - description: (str) 메타 테이블 설명
        - schema: (dict) 메타 아이템 스키마
        - items: (list) 포함된 메타 아이템 리스트
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시
        - user: (str) 메타 테이블 소유 계정명

        ## Example

        ```
        all_my_meta_tables = meta_table_client.list_meta_tables()
        ```
        """

        response = self._request(method="GET", url=f"{MLS_META_API_URL}", params=kwargs).results

        return [MetaTable(**meta_table) for meta_table in response]

    def meta_table_exists(self, id: int = None, name: str = None) -> bool:
        """
        메타 테이블이 존재하는 지 확인합니다.

        ## Args: `id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - id: (optional) (int) 메타 테이블 고유 ID
        - name: (optional) (str) 메타 테이블 이름

        ## Returns
        `bool`

        ## Example

        ```
        meta_table_client.meta_table_exists(name="my_table")
        ```
        """
        assert id or name, "`id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        meta_tables = self.list_meta_tables(id=id, name=name)
        if len(meta_tables) == 0:
            return False
        return True

    def delete_meta_table(self, meta_table: MetaTable) -> MLSResponse:
        """
        메타 테이블을 삭제합니다.

        ## Args

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 테이블 객체

        ## Returns
        `sktmls.MLSResponse`

        ## Example

        ```
        meta_table_client.delete_meta_table(meta_table)
        ```
        """
        assert type(meta_table) == MetaTable

        return self._request(method="DELETE", url=f"{MLS_META_API_URL}/{meta_table.id}")

    def create_meta_item(self, meta_table: MetaTable, item_name: str, item_dict: Dict[str, Any]) -> MetaItem:
        """
        메타 아이템을 생성합니다.

        ## Args

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 아이템이 포함될 메타 테이블 객체
        - item_name: (str) 메타 아이템 이름
        - item_dict: (dict) 메타 아이템 정보

        ## Returns
        `sktmls.meta_tables.MetaItem`

        - id: (int) 메타 아이템 고유 ID
        - name: (str) 메타 아이템 이름
        - values: (dict) 메타 아이템 정보

        ## Example

        ```
        my_meta_item1 = meta_table_client.create_meta_item(
            meta_table=my_meta_table,
            item_name="my_meta_item1",
            item_dict={
                "meta_item_schema1": "string_value",
                "meta_item_schema2": 100
            }
        )
        ```
        """
        assert type(meta_table) == MetaTable

        data = {"name": item_name, "values": item_dict}

        return MetaItem(
            **self._request(method="POST", url=f"{MLS_META_API_URL}/{meta_table.id}/meta_items", data=data).results
        )

    def update_meta_item(
        self, meta_table: MetaTable, meta_item: MetaItem, item_name: str = None, item_dict: Dict[str, Any] = None,
    ) -> MetaItem:
        """
        메타 아이템을 수정합니다.

        ## Args

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 테이블 객체
        - meta_item: (`sktmls.meta_tables.MetaItem`) 메타 아이템 객체
        - item_name: (optional) (str) 메타 아이템 이름
        - item_dict: (optional) (dict) 메타 아이템 정보

        ## Returns:
        `sktmls.meta_tables.MetaItem`

        - id: (int) 메타 아이템 고유 ID
        - name: (str) 메타 아이템 이름
        - values: (dict) 메타 아이템 정보

        ## Example

        ```
        my_meta_item1 = meta_table_client.update_meta_item(
            meta_table=my_meta_table,
            meta_item=my_meta_item1,
            item_name="my_meta_item1",
            item_dict={
                "meta_item_schema1": "another_string_value",
                "meta_item_schema2": 200
            }
        )
        ```
        """
        assert type(meta_table) == MetaTable
        assert type(meta_item) == MetaItem

        data = meta_item.get()
        if item_name:
            data["name"] = item_name
        if item_dict:
            data["values"] = item_dict

        meta_item.reset(
            **self._request(
                method="PUT", url=f"{MLS_META_API_URL}/{meta_table.id}/meta_items/{meta_item.id}", data=data
            ).results
        )

        return meta_item

    def list_meta_items(self, meta_table: MetaTable, **kwargs) -> List[MetaItem]:
        """
        메타 테이블에 포함된 메타 아이템 리스트를 가져옵니다.

        ## Args

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 테이블 객체
        - kwargs: (optional) (dict) 필터 조건
          - id: (int) 메타 아이템 고유 ID
          - name: (str) 메타 아이템 이름

        ## Returns
        list(`sktmls.meta_tables.MetaItem`)

        - id: (int) 메타 아이템 고유 ID
        - name: (str) 메타 아이템 이름
        - values: (dict) 메타 아이템 정보

        ## Example

        ```
        all_meta_items_in_table = meta_table_client.list_meta_items(meta_table=my_meta_table)
        ```
        """
        assert type(meta_table) == MetaTable

        response = self._request(
            method="GET", url=f"{MLS_META_API_URL}/{meta_table.id}/meta_items", params=kwargs
        ).results

        return [MetaItem(**meta_item) for meta_item in response]

    def get_meta_item(self, meta_table: MetaTable, id: int = None, name: str = None) -> MetaItem:
        """
        메타 아이템 정보를 가져옵니다.

        ## Args: `id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 테이블 객체
        - id: (optional) (int) 메타 아이템 고유 ID
        - name: (optional) (str) 메타 아이템 이름

        ## Returns
        `sktmls.meta_tables.MetaItem`

        - id: (int) 메타 아이템 고유 ID
        - name: (str) 메타 아이템 이름
        - values: (dict) 메타 아이템 정보

        ## Example

        ```
        meta_item_by_id = meta_table_client.get_meta_item(meta_table=my_meta_table, id=10)
        meta_item_by_name = meta_table_client.get_meta_item(meta_table=my_meta_table, name="my_item")
        ```
        """
        assert type(meta_table) == MetaTable
        assert id or name, "`id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        meta_items = self.list_meta_item(meta_table, id=id, name=name)
        if len(meta_items) == 0:
            raise MLSClientError(code=404, msg="메타 아이템이 없습니다.")
        elif len(meta_items) > 1:
            raise MLSClientError(code=409, msg="메타 아이템이 여러개 존재합니다.")
        return meta_items[0]

    def delete_meta_item(self, meta_table: MetaTable, meta_item: MetaItem) -> MLSResponse:
        """
        메타 아이템을 메타 테이블에서 삭제합니다.

        ## Args

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 테이블 객체
        - meta_item: (`sktmls.meta_tables.MetaItem`) 메타 아이템 객체

        ## Returns
        `sktmls.MLSResponse`

        ## Example

        ```
        meta_table_client.delete_meta_item(meta_table=my_meta_table, meta_item=my_meta_item)
        ```
        """
        assert type(meta_table) == MetaTable
        assert type(meta_item) == MetaItem

        return self._request(method="DELETE", url=f"{MLS_META_API_URL}/{meta_table.id}/meta_items/{meta_item.id}")

    def create_meta_items(self, meta_table: MetaTable, items_dict: Dict[str, Any]) -> List[MetaItem]:
        """
        메타 아이템 여러 개를 생성/수정 합니다. 추가 할 `item_name`이 존재 시 수정, 미존재 시 생성 합니다.

        ## Args

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 아이템이 포함될 메타 테이블 객체
        - items_dict: (dict) Bulk로 추가할 메타 아이템 정보

        ## Returns
        list(`sktmls.meta_tables.MetaItem`)

        - id: (int) 메타 아이템 고유 ID
        - name: (str) 메타 아이템 이름
        - values: (dict) 메타 아이템 정보

        ## Example

        ```
        my_meta_item1 = meta_table_client.create_meta_item(
            meta_table=my_meta_table,
            items_dict={
                "item_1" : {
                    "value_1" : "a1", "value_2" : "b1"
                },
                "item_2" : {
                    "value_1" : "a2", "value_2" : "b2"
                },
                "item_3" : {
                    "value_1" : "a3", "value_2" : "b3"
                }
            }
        )
        ```
        """
        assert type(meta_table) == MetaTable
        assert len(items_dict) <= 1000, "Bulk로 추가할 meta_items은 한번에 최대 1,000건 까지 지원합니다."

        data = {"items": items_dict}

        response = self._request(method="POST", url=f"{MLS_META_API_URL}/{meta_table.id}/bulk", data=data).results
        return [MetaItem(**meta_item) for meta_item in response]

    def create_meta_items_by_dataframe(
        self, meta_table: MetaTable, meta_items: pandas.core.frame.DataFrame, item_name: str
    ) -> List[MetaItem]:
        """
        메타 아이템 여러 개를 Pandas DataFrame을 이용하여 생성/수정 합니다. 추가 할 `item_name`이 존재 시 수정, 미존재 시 생성 합니다.

        ## Args

        - meta_table: (`sktmls.meta_tables.MetaTable`) 메타 아이템이 포함될 메타 테이블 객체
        - meta_items: (`pandas.core.frame.DataFrame`) Bulk로 추가할 메타 아이템 Pandas DataFrame
        - item_name: (str) meta_items 컬럼 중 meta_item의 이름이 될 컬럼명

        ## Returns
        list(`sktmls.meta_tables.MetaItem`)

        - id: (int) 메타 아이템 고유 ID
        - name: (str) 메타 아이템 이름
        - values: (dict) 메타 아이템 정보

        ## Example

        ```
        my_meta_table = meta_table_client.get_meta_table(name="my_meta_table")

        meta_items = pandas.DataFrame({"name": ["item_1", "item_2", "item_3"], "value_1": ["a1", "a2", "a3"], "value_2": ["b1", "b2", "b3"]})

        my_meta_items = meta_table_client.create_meta_item(
            meta_table=my_meta_table,
            meta_items=meta_items,
            item_name="name"
        )
        ```
        """
        assert type(meta_table) == MetaTable
        assert type(meta_items) == pandas.core.frame.DataFrame
        assert meta_items.shape[0] <= 1000, "Bulk로 추가할 meta_items은 한번에 최대 1,000건 까지 지원합니다."

        meta_items.set_index([item_name], drop=True, inplace=True)
        items_dict = meta_items.to_dict(orient="index")

        data = {"items": items_dict}

        response = self._request(method="POST", url=f"{MLS_META_API_URL}/{meta_table.id}/bulk", data=data).results
        return [MetaItem(**meta_item) for meta_item in response]



from typing import List

from dateutil import parser

from sktmls import MLSClient, MLSENV, MLSResponse, MLSClientError

MLS_DIMENSIONS_API_URL = "/api/v1/dimensions"


class Dimension:
    """
    MLS 디멘전 클래스입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) 디멘전 고유 ID
            - name: (str) 디멘전 이름
            - dimension_type: (str) 디멘전 타입 (`user`|`item`)
            - data_type: (str) 디멘전 데이터 타입 (`string`|`number`)
            - description: (str) 디멘전 설명
            - default: (str) 디멘전 기본 값
            - task_id: (str) 디멘전 생성 Airflow Task ID
            - period: (str) 디멘전 생성 주기 (`daily`|`monthly`)
            - is_derivative: (bool) 디멘전 파생변수 여부
            - created_at: (datetime) 생성일시
            - updated_at: (datetime) 수정일시

        ## Returns
        `sktmls.dimensions.Dimension`
        """
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.dimension_type = kwargs.get("dimension_type")
        self.data_type = kwargs.get("data_type")
        self.description = kwargs.get("description")
        self.default = kwargs.get("default")
        self.task_id = kwargs.get("task_id")
        self.period = kwargs.get("period")
        self.is_derivative = kwargs.get("is_derivative")
        self.created_at = parser.parse(kwargs.get("created_at"))
        self.updated_at = parser.parse(kwargs.get("updated_at"))

    def get(self):
        return self.__dict__

    def reset(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class DimensionClient(MLSClient):
    """
    MLS 디멘전 관련 기능들을 제공하는 클라이언트입니다.
    """

    def __init__(
        self, env: MLSENV = None, username: str = None, password: str = None,
    ):
        """
        ## Args

        - env: (`sktmls.MLSENV`) 접근할 MLS 환경 (`sktmls.MLSENV.DEV`|`sktmls.MLSENV.STG`|`sktmls.MLSENV.PRD`) (기본값: `sktmls.MLSENV.STG`)
        - username: (str) MLS 계정명 (기본값: $MLS_USERNAME)
        - password: (str) MLS 계정 비밀번호 (기본값: $MLS_PASSWORD)

        ## Returns
        `sktmls.dimensions.DimensionClient`

        ## Example

        ```
        dimension_client = DimensionClient(env=MLSENV.STG, username="mls_account", password="mls_password")
        ```
        """

        super().__init__(env=env, username=username, password=password)

    def create_dimension(
        self,
        name: str,
        dimension_type: str,
        data_type: str,
        default: str,
        period: str,
        description: str = None,
        task_id: str = None,
        is_derivative: bool = None,
    ) -> Dimension:
        """
        디멘전을 생성합니다.

        ## Args

        - name: (str) 디멘전 이름
        - dimension_type: (str) 디멘전 타입 (`user`|`item`)
        - data_type: (str) 디멘전 데이터 타입 (`string`|`number`|`array`|`json`)
        - default: (str) 디멘전 기본 값
        - period: (str) 디멘전 생성 주기 (`daily`|`monthly`)
        - description: (optional) (str) 디멘전 설명
        - task_id: (optional) (str) 디멘전 생성 Airflow Task ID
        - is_derivative: (optional) (bool) 디멘전 파생변수 여부

        ## Returns
        `sktmls.dimensions.Dimension`

        - id: (int) 디멘전 고유 ID
        - name: (str) 디멘전 이름
        - dimension_type: (str) 디멘전 타입 (`user`|`item`)
        - data_type: (str) 디멘전 데이터 타입 (`string`|`number`|`array`|`json`)
        - description: (str) 디멘전 설명
        - default: (str) 디멘전 기본 값
        - task_id: (str) 디멘전 생성 Airflow Task ID
        - period: (str) 디멘전 생성 주기 (`daily`|`monthly`)
        - is_derivative: (bool) 디멘전 파생변수 여부
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```
        dimension = dimension_client.create_dimension(
            name="hello",
            dimension_type="user",
            data_type="string",
            default="1",
            period="daily"
        )
        ```
        """
        assert dimension_type in ["user", "item"]
        assert data_type in ["string", "number", "array", "json"]
        assert period in ["daily", "monthly"]

        data = {
            "name": name,
            "dimension_type": dimension_type,
            "data_type": data_type,
            "default": default,
            "period": period,
        }
        if description:
            data["description"] = description
        if task_id:
            data["task_id"] = task_id
        if is_derivative is not None:
            data["is_derivative"] = is_derivative

        return Dimension(**self._request(method="POST", url=MLS_DIMENSIONS_API_URL, data=data).results)

    def update_dimension(
        self,
        dimension: Dimension,
        name: str = None,
        dimension_type: str = None,
        data_type: str = None,
        default: str = None,
        period: str = None,
        description: str = None,
        task_id: str = None,
        is_derivative: bool = None,
    ) -> Dimension:
        """
        디멘전 정보를 생성합니다.

        ## Args

        - dimension: (`sktmls.dimensions.Dimension`) 디멘전 객체
        - name: (optional) (str) 디멘전 이름
        - dimension_type: (optional) (str) 디멘전 타입 (`user`|`item`)
        - data_type: (optional) (str) 디멘전 데이터 타입 (`string`|`number`|`array`|`json`)
        - default: (optional) (str) 디멘전 기본 값
        - period: (optional) (str) 디멘전 생성 주기 (`daily`|`monthly`)
        - description: (optional) (str) 디멘전 설명
        - task_id: (optional) (str) 디멘전 생성 Airflow Task ID
        - is_derivative: (optional) (bool) 디멘전 파생변수 여부

        ## Returns
        `sktmls.dimensions.Dimension`

        - id: (int) 디멘전 고유 ID
        - name: (str) 디멘전 이름
        - dimension_type: (str) 디멘전 타입 (`user`|`item`)
        - data_type: (str) 디멘전 데이터 타입 (`string`|`number`|`array`|`json`)
        - description: (str) 디멘전 설명
        - default: (str) 디멘전 기본 값
        - task_id: (str) 디멘전 생성 Airflow Task ID
        - period: (str) 디멘전 생성 주기 (`daily`|`monthly`)
        - is_derivative: (bool) 디멘전 파생변수 여부
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```
        dimension = dimension_client.update_dimension(
            name="bye",
            default="2",
            period="monthly"
        )
        ```
        """
        assert type(dimension) == Dimension

        data = dimension.get()
        if name:
            data["name"] = name
        if dimension_type:
            assert dimension_type in ["user", "item"]
            data["dimension_type"] = dimension_type
        if data_type:
            assert data_type in ["string", "number", "array", "json"]
            data["data_type"] = data_type
        if default:
            data["default"] = default
        if period:
            assert period in ["daily", "monthly"]
            data["period"] = period
        if description:
            data["description"] = description
        if task_id:
            data["task_id"] = task_id
        if is_derivative is not None:
            data["is_derivative"] = is_derivative

        dimension.reset(
            **self._request(method="PUT", url=f"{MLS_DIMENSIONS_API_URL}/{dimension.id}", data=data).results
        )

        return dimension

    def get_dimension(self, dimension_type: str, id: int = None, name: str = None) -> Dimension:
        """
        디멘전 정보를 가져옵니다.

        ## Args: `id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - dimension_type: (str) 디멘전 타입 (`user`|`item`)
        - id: (optional) (int) 디멘전 고유 ID
        - name: (optional) (str) 디멘전 이름


        ## Returns
        `sktmls.dimensions.Dimension`

        - id: (int) 디멘전 고유 ID
        - name: (str) 디멘전 이름
        - dimension_type: (str) 디멘전 타입 (`user`|`item`)
        - data_type: (str) 디멘전 데이터 타입 (`string`|`number`|`array`|`json`)
        - description: (str) 디멘전 설명
        - default: (str) 디멘전 기본 값
        - task_id: (str) 디멘전 생성 Airflow Task ID
        - period: (str) 디멘전 생성 주기 (`daily`|`monthly`)
        - is_derivative: (bool) 디멘전 파생변수 여부
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```
        dimension = dimension_client.get_dimension(
            dimension_type="user",
            name="hello"
        )
        ```
        """
        assert id or name, "`id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        dimensions = self.list_dimensions(dimension_type=dimension_type, id=id, name=name)
        if len(dimensions) == 0:
            raise MLSClientError(code=404, msg="디멘전이 없습니다.")
        elif len(dimensions) > 1:
            raise MLSClientError(code=409, msg="디멘전이 여러개 존재합니다.")
        return dimensions[0]

    def list_dimensions(self, dimension_type: str, **kwargs) -> List[Dimension]:
        """
        디멘전 리스트를 가져옵니다.

        ## Args

        - dimension_type: (str) 디멘전 타입 (`user`|`item`)
        - kwargs: (optional) (dict) 필터 조건
            - id: (int) 디멘전 고유 ID
            - name: (str) 디멘전 이름
            - data_type: (str) 디멘전 데이터 타입 (`string`|`number`|`array`|`json`)
            - task_id: (str) 디멘전 생성 Airflow Task ID
            - period: (str) 디멘전 생성 주기 (`daily`|`monthly`)
            - is_derivative: (bool) 파생 변수 여부


        ## Returns
        list(`sktmls.dimensions.Dimension`)

        - id: (int) 디멘전 고유 ID
        - name: (str) 디멘전 이름
        - dimension_type: (str) 디멘전 타입 (`user`|`item`)
        - data_type: (str) 디멘전 데이터 타입 (`string`|`number`|`array`|`json`)
        - description: (str) 디멘전 설명
        - default: (str) 디멘전 기본 값
        - task_id: (str) 디멘전 생성 Airflow Task ID
        - period: (str) 디멘전 생성 주기 (`daily`|`monthly`)
        - is_derivative: (bool) 디멘전 파생변수 여부
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```
        dimensions = dimension_client.list_dimensions(
            dimension_type="user"
        )

        derivative_dimensions = dimension_client.list_dimensions(
            dimension_type="user",
            is_derivative=True
        )

        string_dimensions = dimension_client.list_dimensions(
            dimension_type="user",
            data_type="string"
        )
        ```
        """

        response = self._request(
            method="GET", url=f"{MLS_DIMENSIONS_API_URL}?type={dimension_type}", params=kwargs
        ).results

        return [Dimension(**dimension) for dimension in response]

    def delete_dimension(self, dimension: Dimension) -> MLSResponse:
        """
        디멘전을 삭제합니다.

        ## Args

        - dimension: (`sktmls.dimensions.Dimension`) 디멘전 객체

        ## Returns
        `sktmls.MLSResponse`

        ## Example

        ```
        dimension_client.delete_dimension(dimension)
        ```
        """

        assert type(dimension) == Dimension

        return self._request(method="DELETE", url=f"{MLS_DIMENSIONS_API_URL}/{dimension.id}")



from typing import Any, Dict, List

from sktmls.models import MLSModelError, MLSRuleModel


class InfoUnpaidRuleModel(MLSRuleModel):
    def predict(self, x: List[Any]) -> Dict[str, Any]:
        if len(self.features) != len(x):
            raise MLSModelError("The length of input is different from that of features in model.json")

        filter_col = x[0]

        if filter_col != "Y":
            return {"items": []}
        else:
            return {"items": [{"id": "INF0000001", "name": "미납 요금 및 납부 안내", "type": "info_unpaid", "props": {}}]}



import setuptools
import os

SKTMLS_VERSION = os.getenv("SKTMLS_VERSION")
if not SKTMLS_VERSION:
    raise TypeError("NO SKTMLS_VERSION")


def load_long_description():
    with open("README.md", "r") as f:
        long_description = f.read()
    return long_description


setuptools.setup(
    name="sktmls",
    version=SKTMLS_VERSION,
    author="Colin",
    author_email="colin@sktai.io",
    description="A package for MLS-Models",
    long_description=load_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/sktaiflow/mls-model-registry",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        "joblib==0.14.1",
        "lightgbm==2.3.1",
        "numpy==1.18.3",
        "pandas==1.0.3",
        "pdoc3==0.8.4",
        "pytest==5.4.2",
        "pytz==2020.1",
        "requests==2.24.0",
        "scikit-learn==0.22.2.post1",
        "scipy==1.4.1",
        "xgboost==1.0.2",
        "python-dateutil==2.8.1",
    ],
)



MLS_API_URL = {
    "dev": "https://ab-onprem.dev.sktmls.com",
    "stg": "https://ab-onprem.stg.sktmls.com",
    "prd": "https://ab-onprem.sktmls.com",
}

HDFS_OPTIONS = """-Dfs.s3a.proxy.host=awsproxy.datalake.net \
                 -Dfs.s3a.proxy.port=3128 \
                 -Dfs.s3a.endpoint=s3.ap-northeast-2.amazonaws.com \
                 -Dfs.s3a.security.credential.provider.path=jceks:///user/tairflow/s3_mls.jceks \
                 -Dfs.s3a.fast.upload=true -Dfs.s3a.acl.default=BucketOwnerFullControl"""



import pytest

from sktmls.models.contrib import BnfLoyaltyRuleModel


class TestBnfLoyaltyRuleModels:
    @pytest.fixture(scope="class")
    def param(self):
        return {"model": BnfLoyaltyRuleModel("test_model", "test_version", ["svc_scrb_period"])}

    def test_000_bnf_loyalty_return(self, param):
        assert param.get("model").predict([0]) == {"items": []}

    def test_001_bnf_loyalty_return(self, param):
        assert param.get("model").predict([800]) == {
            "items": [{"id": "BNF0000003", "name": "장기 고객 혜택 정보", "type": "bnf_loyalty", "props": {}}]
        }



import base64
import json
import os
import requests
from typing import Any

from sktmls import MLSENV
from sktmls.config import config


class MLSClientError(Exception):
    def __init__(self, code: int, msg: str):
        super().__init__(msg)
        self.code = code
        self.msg = msg


class MLSResponse:
    def __init__(self, response: requests.models.Response):
        data = json.loads(response.content.decode("utf-8"))
        self.status = response.status_code
        self.code = data.get("code")
        self.error = data.get("error")
        self.results = data.get("results")

        if self.error:
            raise MLSClientError(code=self.code, msg=self.error)


class MLSClient:
    def __init__(self, env: MLSENV = None, username: str = None, password: str = None):
        if env:
            assert env in MLSENV.list_items(), "Invalid environment."
            self.__env = env
        elif os.environ.get("MLS_ENV"):
            self.__env = os.environ["MLS_ENV"]
        else:
            self.__env = MLSENV.STG

        if username:
            assert type(username) == str, "Invalid type of username"
            self.__username = username
        elif os.environ.get("MLS_USERNAME"):
            self.__username = os.environ["MLS_USERNAME"]
        else:
            raise MLSClientError(
                code=400, msg="'username' must be provided with parameter or environment variable (MLS_USERNAME)"
            )

        if password:
            assert type(password) == str, "Invalid type of password"
            self.__password = password
        elif os.environ.get("MLS_PASSWORD"):
            self.__password = os.environ["MLS_PASSWORD"]
        else:
            raise MLSClientError(
                code=400, msg="'password' must be provided with parameter or environment variable (MLS_PASSWORD)"
            )

        self.__api_token = base64.b64encode(f"{self.__username}:{self.__password}".encode()).decode("utf-8")

    def get_env(self) -> MLSENV:
        return self.__env

    def get_username(self) -> str:
        return self.__username

    def _request(self, method: str, url: str, data: Any = None, params=None) -> MLSResponse:
        response = requests.request(
            method=method,
            url=f"{config.MLS_API_URL[self.__env.value]}{'' if url.startswith('/') else '/'}{url}",
            headers={"Authorization": f"Basic {self.__api_token}"},
            json=data,
            params=params,
        )

        return MLSResponse(response)
